@@ -7,6 +7,7 @@
 
 #include "includes.h"
 #include <linux/kdev_t.h>
+#include <time.h>
 
 struct key root_dir_key = {0, 0, {{0, 0},}};
 struct key parent_root_dir_key = {0, 0, {{0, 0},}};
@@ -160,6 +161,7 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags,
     fs->fs_flags = flags; /* O_RDONLY or O_RDWR */
 
     fs->fs_format = get_reiserfs_format (sb);
+
     
     /*reiserfs_read_bitmap_blocks(fs);*/
     if (flags & O_RDWR)
@@ -181,6 +183,8 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
 				     int new_format)
 {
     reiserfs_filsys_t * fs;
+    unsigned int bmap_nr = reiserfs_bmap_nr(block_count, block_size);;
+    time_t now;
 
 
     /* convert root dir key and parent root dir key to little endian format */
@@ -206,7 +210,7 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
 	return 0;
     }
 
-    fs->fs_dev = open (filename, O_RDWR 
+    fs->fs_dev = open (filename, O_RDWR | O_EXCL
 #if defined(O_LARGEFILE)
 		       | O_LARGEFILE
 #endif
@@ -278,9 +282,16 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
     /* sb_fsck_state */
     /* sb_hash_function_code */
     /* sb_tree_height */
+
     set_sb_bmap_nr (fs->fs_ondisk_sb,
-		    (block_count + (block_size * 8 - 1)) / (block_size * 8));
+                    reiserfs_bmap_over(bmap_nr) ? 0 : bmap_nr);
+
     set_sb_version (fs->fs_ondisk_sb, version);
+    set_sb_v2_lastcheck(fs->fs_ondisk_sb, time(&now));
+    set_sb_v2_check_interval(fs->fs_ondisk_sb, DEFAULT_CHECK_INTERVAL);
+    set_sb_v2_mnt_count(fs->fs_ondisk_sb, 1);
+    set_sb_v2_max_mnt_count(fs->fs_ondisk_sb, DEFAULT_MAX_MNT_COUNT);
+
     /* sb_not_used1 */
 
     mark_buffer_dirty (fs->fs_super_bh);
@@ -1131,6 +1142,19 @@ int create_dir_sd (reiserfs_filsys_t * fs,
     memset(&sd, 0, sizeof(sd));
     make_dir_stat_data (fs->fs_blocksize, key_format, get_key_dirid (key),
 			get_key_objectid (key), &ih, &sd);
+    
+    /* if calling user is not root set the owner of the root entry
+     * to the calling user */
+    if(getuid()) {
+	if (key_format == KEY_FORMAT_1) {
+	    struct stat_data_v1 *sd_v1 = (struct stat_data_v1 *)&sd;
+	    set_sd_v1_uid (sd_v1, getuid());
+	    set_sd_v1_gid (sd_v1, getgid());
+	} else {
+	    set_sd_v2_uid (&sd, getuid());
+	    set_sd_v2_gid (&sd, getgid());
+	}
+    }
 
     if (modify_item)
 	modify_item (&ih, &sd);
@@ -1175,6 +1199,28 @@ void make_sure_root_dir_exists (reiserfs_filsys_t * fs,
     		&parent_root_dir_key, ih_flags);
 }
 
+int block_size_ok (int blocksize, int force)
+{
+    int pagesize = getpagesize();
+    if (blocksize > 4096) {
+        reiserfs_warning (stderr, "Block sizes larger than 4k are not "
+                          "supported on all architectures.\n");
+        if (blocksize > pagesize)
+            reiserfs_warning (stderr, "The newly created filesystem will not "
+                              "be mountable on this system.\n");
+        else
+            reiserfs_warning (stderr, "The newly created filesystem may not "
+                              "be mountable on other systems.\n");
+        check_forcing_ask_confirmation (force);
+    } else if (blocksize < 4096) {
+        reiserfs_warning (stderr, "Block sizes smaller than 4k "
+			  "are not supported.\n");
+	return 0;
+    }
+
+    return 1;
+}
+
 
 /* we only can use a file for filesystem or journal if it is either not
    mounted block device or regular file and we are forced to use it */
